////////////////////////////////////////////////////////////////////////////
//	256BMP.CPP
////////////////////////////////////////////////////////////////////////////

#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include <owl\gdiobjec.h>
#include <owl\color.h>
#pragma hdrstop
#include "256bmp.rh"

#define BITMAPFILE "scream.bmp"


///////////////////////////////////////////////////////////////////////////
class T256bmpWin: public TFrameWindow {
public:
	T256bmpWin(TWindow* parent, const char far* title);
	~T256bmpWin();
protected:
	void Paint(TDC& dc, bool erase, TRect& rect);
	void EvSize(UINT sizeType, TSize& size);
private:
	TBitmap *bitmap;
	TPalette *palette;
DECLARE_RESPONSE_TABLE(T256bmpWin);
};

DEFINE_RESPONSE_TABLE1(T256bmpWin, TFrameWindow)
	EV_WM_SIZE,
END_RESPONSE_TABLE;


T256bmpWin::T256bmpWin(TWindow* parent, const char far* title)
	: TFrameWindow(parent, title),
		TWindow(parent, title)
{
	AssignMenu(ID_MENU);

	Attr.X = 100;
	Attr.Y = 50;
	Attr.W = 300;
	Attr.H = 300;

	TDib *dib = new TDib(*GetModule(), TResId(BITMAP_1));	//make a dib from resource
	palette = new TPalette(*dib);									//make a palette from dib
	bitmap = new TBitmap(*dib, palette);					   //bitmap with given palette
	delete dib;

//	if you want to read a bitmap file from the disk
//	TDib *dib = new TDib(BITMAPFILE);   //make a dib from .BMP
//	palette = new TPalette(*dib);				//make a palette from dib
//	bitmap = new TBitmap(*dib);
//	delete dib;
}


T256bmpWin::~T256bmpWin()
{
	delete bitmap;
	delete palette;
}


void T256bmpWin::Paint(TDC& dc, bool /*erase*/, TRect& /*rect*/)
{
//	dc.SetStretchBltMode(COLORONCOLOR);		//if you are using StretchBlt
	dc.SelectObject(*palette);		//select the palette
	dc.RealizePalette();					//and match the colors

	TMemoryDC memdc(dc);
	memdc.SelectObject(*bitmap);
	memdc.SelectObject(*palette);

	TRect rect = GetClientRect();
	int bmpWidth = bitmap->Width();
	int bmpHeight = bitmap->Height();
	int x = (rect.Width() - bmpWidth) / 2;
	int y = (rect.Height() - bmpHeight) / 2;

	dc.BitBlt(x, y, bmpWidth, bmpHeight,	//destination
		memdc, 0, 0, SRCCOPY);
	dc.RestorePalette();
	memdc.RestorePalette();
}


void T256bmpWin::EvSize(UINT sizeType, TSize& size)
{
	TFrameWindow::EvSize(sizeType, size);
	Invalidate();
	UpdateWindow();
}   

///////////////////////////////////////////////////////////////////////////
class TFadeApp: public TApplication {
public:
	TFadeApp(const char far* name)
		: TApplication(name) {};
	void InitMainWindow();
};


void TFadeApp::InitMainWindow()
{
	MainWindow = new T256bmpWin(0, "256-color Bitmap");
}


int OwlMain(int /*argc*/, char* /*argv*/[])
{
	TFadeApp app("256-color Bitmap");
	return app.Run();
}

